@@ -14,34 +14,37 @@
 typedef struct SoupAddress           SoupAddress;
 typedef struct SoupConnection        SoupConnection;
 typedef struct SoupMessage           SoupMessage;
+typedef struct SoupMessageFilter     SoupMessageFilter;
 typedef struct SoupServer            SoupServer;
 typedef union  SoupServerAuth        SoupServerAuth;
 typedef struct SoupServerAuthContext SoupServerAuthContext;
 typedef struct SoupServerMessage     SoupServerMessage;
 typedef struct SoupSession           SoupSession;
+typedef struct SoupSessionAsync      SoupSessionAsync;
+typedef struct SoupSessionSync       SoupSessionSync;
 typedef struct SoupSocket            SoupSocket;
 typedef struct SoupUri               SoupUri;
 
-#define SOUP_MAKE_TYPE(l,t,ci,i,parent) \
-GType l##_get_type(void)\
+#define SOUP_MAKE_INTERFACE(type_name,TypeName,base_init) \
+GType type_name##_get_type(void)\
 {\
 	static GType type = 0;				\
 	if (!type){					\
 		static GTypeInfo const object_info = {	\
-			sizeof (t##Class),		\
+			sizeof (TypeName##Class),	\
 							\
-			(GBaseInitFunc) NULL,		\
+			(GBaseInitFunc) base_init,	\
 			(GBaseFinalizeFunc) NULL,	\
 							\
-			(GClassInitFunc) ci,		\
+			(GClassInitFunc) NULL,		\
 			(GClassFinalizeFunc) NULL,	\
 			NULL,	/* class_data */	\
 							\
-			sizeof (t),			\
+			0,				\
 			0,	/* n_preallocs */	\
-			(GInstanceInitFunc) i,		\
+			(GInstanceInitFunc) NULL,	\
 		};					\
-		type = g_type_register_static (parent, #t, &object_info, 0); \
+		type = g_type_register_static (G_TYPE_INTERFACE, #TypeName, &object_info, 0); \
 	}						\
 	return type;					\
 }
